// --- [ rotationsblinker.c ] -------------------------------------------------
//
// Rotationsblinker mit bis zu 5 LEDs am Attiny 25.
// Simuliert mechanische Rotationsblinker mit geringfgigen Abweichungen
// der Drehgeschwindigkeiten
//
// ----------------------------------------------------------------------------

#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>

#define ANZAHL_LED		5
				
#define DELTA				0	
#define LED0_LIMIT		255
#define LED1_LIMIT		LED0_LIMIT - (3  + DELTA)
#define LED2_LIMIT		LED1_LIMIT - (7  + DELTA)
#define LED3_LIMIT		LED2_LIMIT - (11 + DELTA)
#define LED4_LIMIT		LED3_LIMIT - (13 + DELTA)

#define LED_PORT			PORTB			// LED-Ports
#define LED0_PIN			PB0
#define LED1_PIN			PB1
#define LED2_PIN			PB2
#define LED3_PIN			PB3
#define LED4_PIN			PB4

#define COMPARE1A			100			// gibt die "Dreh-Geschwindigkeit" vor
												// als Feintuning, je kleiner, desto schneller
												// die grobe Einstellung wird durch TCCR1 vorgenommen
#define TRUE				1
#define FALSE				0

// Globale Variable
volatile uint8_t  flag;					// Flags fr Timer_ovf

// ----------------------------------------------------------------------------
int main( void )
{
uint8_t led[ANZAHL_LED];				// Array mit je einem Counter fr jede LED
uint8_t i;

ACSR = (1<<ACD);							// Analog Comparator Disable
PRR  = (1<<PRTIM0 | 1<<PRUSI);		// Stromsparmodus, Timer0 und USI ausschalten

DDRB  = (1<<LED0_PIN | 1<<LED1_PIN |1<<LED2_PIN | 1<<LED3_PIN | 1<<LED4_PIN);

// --- TIMER1 -----------------------------------------------------------------

TCCR1 = ( 0<<CS13| 1<<CS12| 1<<CS11 | 0<<CS10);	// Prescale = 32

OCR1A  = COMPARE1A;						// Compare-Wert 
TIMSK |= (1<<OCIE1A);					// Compar1a Interrupt enablen

CLKPR = (1<<CLKPCE);						// Clock-division-faktor auf 1 setzen (default=8 -> 1MHz))
CLKPR = 0;
asm volatile("sei");	 					// enable_interrupt


while(1)
	{
	while (!(flag));						// warten, bis Flag von Compare1a gesetzt wird
	flag = FALSE;
	
	for (i=0; i < ANZAHL_LED; i++)	// die Counter aller Ausgnge incrementieren
		{
		led[i]++;
		}
		
	if (led[0] >= LED0_LIMIT)			// und dann testen, wer sein Limit berschritten hat
		{
		led[0] = 0;							// den Counter zurcksetzen
		LED_PORT ^= (1<<LED0_PIN);		// wenn Limit erreicht, dann togglen 
		}
	
	if (led[1] >= LED1_LIMIT)	
		{
		led[1] = 0;
		LED_PORT ^= (1<<LED1_PIN);
		}

	if (led[2] >= LED2_LIMIT)	
		{
		led[2] = 0;
		LED_PORT ^= (1<<LED2_PIN);
		}		
	
	if (led[3] >= LED3_LIMIT)	
		{
		led[3]    = 0;
		LED_PORT ^= (1<<LED3_PIN);
		}	
	
	if (led[4] >= LED2_LIMIT)	
		{
		led[4] = 0;
		LED_PORT ^= (1<<LED4_PIN);
		}	
	}
}


// ----------------------------------------------------------------------------
// bestimmt die Zeitdauer eines Signals bzw. einer Pause,
// OCR1A legt das Zeitintervall fest.

ISR(TIMER1_COMPA_vect)
{

TCNT1 = 0;								// den Counter zurcksetzen
flag  = TRUE;							// veranlasst das Incrementieren der Zhler

}

// --- [ eof ] ----------------------------------------------------------------
